@@ -11,7 +11,6 @@ import asyncio
 import contextlib
 import time
 from collections.abc import AsyncGenerator, Callable, Sequence
-from copy import copy
 from dataclasses import dataclass, field
 from typing import Unpack
 
@@ -24,7 +23,6 @@ from flexai.message import (
     MessageContent,
     SystemMessage,
     TextBlock,
-    ThoughtBlock,
     ToolCall,
     ToolResult,
     ToolUseChunk,
@@ -70,6 +68,14 @@ class Agent:
     # The mapping of tool names to tool functions.
     toolbox: dict[str, Tool] = field(default_factory=dict, init=False)
 
+    # Queue for streaming tool item results.
+    tool_item_queue: asyncio.Queue = field(default_factory=asyncio.Queue, init=False)
+
+    # Event to signal whether the queue is empty.
+    tool_item_queue_is_not_empty: asyncio.Event = field(
+        default_factory=asyncio.Event, init=False
+    )
+
     def __post_init__(self):
         """Perform post-initialization setup.
 
@@ -95,8 +101,8 @@ class Agent:
             capability.setup(self)
 
     async def modify_messages(
-        self, messages: list[Message]
-    ) -> AsyncGenerator[MessageContent | Message | list[Message], None]:
+        self, messages: Sequence[Message]
+    ) -> AsyncGenerator[MessageContent | Message | Sequence[Message], None]:
         """Hook to modify the messages before sending them to the LLM.
 
         Args:
@@ -111,7 +117,7 @@ class Agent:
         # Iterate through the capabilities and modify the messages.
         for capability in self.capabilities:
             modified_messages = None
-            async for output in capability.modify_messages(copy(messages)):
+            async for output in capability.modify_messages(messages):
                 # This is a partial message chunk.
                 if isinstance(output, (MessageContent, Message)):
                     yield output
@@ -178,7 +184,7 @@ class Agent:
         yield tools
 
     async def modify_response(
-        self, messages: list[Message], response: AIMessage
+        self, messages: Sequence[Message], response: AIMessage
     ) -> AsyncGenerator[MessageContent | AIMessage, None]:
         """Hook to modify the AI response before sending it to the user.
 
@@ -217,22 +223,21 @@ class Agent:
         yield response
 
     async def invoke_tool(
-        self, tool_call: ToolCall, messages: list[Message]
-    ) -> AsyncGenerator[MessageContent | Message, None]:
+        self, tool_call: ToolCall, messages: Sequence[Message]
+    ) -> ToolResult:
         """Execute a tool and yield progress before returning the final result.
 
         Args:
             tool_call: The tool call to execute.
             messages: The current conversation messages.
 
-        Yields:
-            Intermediate message chunks followed by the final tool result.
-
+        Returns:
+            The final ToolResult after execution.
         """
         try:
             tool = self.toolbox[tool_call.name]
         except KeyError:
-            yield ToolResult(
+            return ToolResult(
                 tool_call_id=tool_call.id,
                 result=TextBlock(
                     text=f"The tool `{tool_call.name}` is not available to you."
@@ -240,7 +245,6 @@ class Agent:
                 execution_time=0,
                 is_error=True,
             )
-            return
 
         is_error = False
 
@@ -265,14 +269,17 @@ class Agent:
                 async with contextlib.aclosing(result) as result:
                     async for intermediate in result:
                         if last_received is not None:
-                            yield ToolUseChunk(
+                            chunk_to_yield = ToolUseChunk(
                                 id=tool_call.id,
                                 tool_name=tool_call.name,
                                 content=last_received,
                             )
+                            await self.tool_item_queue.put(chunk_to_yield)
+                            self.tool_item_queue_is_not_empty.set()
                             last_received = None
                         if isinstance(intermediate, (Message, ToolResult)):
-                            yield intermediate
+                            await self.tool_item_queue.put(intermediate)
+                            self.tool_item_queue_is_not_empty.set()
                             continue
                         last_received = intermediate
                     result = last_received
@@ -283,18 +290,86 @@ class Agent:
 
         end_time = time.monotonic()
 
-        yield ToolResult(
+        return ToolResult(
             tool_call_id=tool_call.id,
             result=result,
             execution_time=end_time - start_time,
             is_error=is_error,
         )
 
+    async def invoke_tools(
+        self, tool_calls: Sequence[ToolCall], messages: Sequence[Message]
+    ) -> AsyncGenerator[ToolUseChunk | Message | Sequence[ToolResult], None]:
+        """Invoke multiple tools asynchronously and yield their results as they complete.
+
+        Args:
+            tool_calls: The list of tool calls to execute.
+            messages: The current conversation messages.
+
+        Yields:
+            ToolUseChunk or Message: Intermediate tool use chunks or messages.
+            Sequence[ToolResult]: Final list of tool results after all tools have completed.
+        """
+        tool_tasks = [
+            asyncio.create_task(self.invoke_tool(tool_call, messages))
+            for tool_call in tool_calls
+        ]
+        remaining_tasks = set(tool_tasks)
+
+        async def wait_for_queue_item():
+            await self.tool_item_queue_is_not_empty.wait()
+
+        try:
+            while remaining_tasks:
+                wait_for_queue = asyncio.create_task(wait_for_queue_item())
+                done, _ = await asyncio.wait(
+                    {wait_for_queue, *remaining_tasks},
+                    return_when=asyncio.FIRST_COMPLETED,
+                )
+                if wait_for_queue in done:
+                    item = await self.tool_item_queue.get()
+                    if self.tool_item_queue.empty():
+                        self.tool_item_queue_is_not_empty.clear()
+                    yield item
+                else:
+                    # Cancel the wait_for_queue task if a tool task completed first
+                    wait_for_queue.cancel()
+                    with contextlib.suppress(asyncio.CancelledError):
+                        await wait_for_queue
+
+                # All tool tasks that were *just* completed should not be waited for anymore.
+                remaining_tasks -= done
+
+        finally:
+            # Yield any remaining items in the queue
+            while not self.tool_item_queue.empty():
+                item = await self.tool_item_queue.get()
+                yield item
+            self.tool_item_queue_is_not_empty.clear()
+            task_results = await asyncio.gather(*tool_tasks, return_exceptions=True)
+
+        # Collect all results after they finish
+        tool_results = []
+        for idx in range(len(tool_calls)):
+            if isinstance(task_results[idx], Exception):
+                tool_call = tool_calls[idx]
+                tool_results.append(
+                    ToolResult(
+                        tool_call_id=tool_call.id,
+                        result=f"({tool_call.name}) {task_results[idx]!s}",
+                        is_error=True,
+                    )
+                )
+            else:
+                tool_results.append(task_results[idx])
+
+        yield tool_results
+
     async def modify_tool_results(
         self,
-        messages: list[Message],
+        messages: Sequence[Message],
         tool_call_message: Message[Sequence[MessageContent] | str],
-        tool_results: list[MessageContent],
+        tool_results: Sequence[MessageContent],
     ) -> AsyncGenerator[Message | MessageContent, None]:
         """Hook to modify the tool calls and results before sending them to the user.
 
@@ -316,7 +391,7 @@ class Agent:
         # Iterate through the capabilities and modify the tool results.
         for capability in self.capabilities:
             async for output in capability.modify_tool_results(
-                messages=copy(messages),
+                messages=messages,
                 tool_call_message=tool_call_message,
                 tool_results=tool_results,
             ):
@@ -324,7 +399,7 @@ class Agent:
 
     async def _get_chat_response(
         self,
-        messages: list[Message],
+        messages: Sequence[Message],
         system: str | SystemMessage,
         tools: Sequence[Tool],
         current_llm_index: int = 0,
@@ -388,7 +463,7 @@ class Agent:
 
     async def _stream_chat_response(
         self,
-        messages: list[Message],
+        messages: Sequence[Message],
         system: str | SystemMessage,
         tools: Sequence[Tool],
         current_llm_index: int = 0,
@@ -456,11 +531,11 @@ class Agent:
 
     async def step(
         self,
-        messages: list[Message],
+        messages: Sequence[Message],
         stream: bool,
         llm_exception_callback: Callable[[Exception, int], bool] | None = None,
         **run_args: Unpack[PartialAgentRunArgs],
-    ) -> AsyncGenerator[MessageContent | Message | list[Message], None]:
+    ) -> AsyncGenerator[MessageContent | Message | Sequence[Message], None]:
         """Process a single turn in the conversation.
 
         Generates a response using the language model and determines if any
@@ -581,7 +656,7 @@ class Agent:
             )
         ):
             if response.content[0].name == "send_message":
-                response.content = response.content[0].input["message"]
+                response = response.with_content(response.content[0].input["message"])
             yield response
             return
 
@@ -590,7 +665,7 @@ class Agent:
 
     async def run(
         self,
-        messages: list[Message],
+        messages: Sequence[Message],
         stream: bool = False,
         llm_exception_callback: Callable[[Exception, int], bool] | None = None,
         **run_args: Unpack[PartialAgentRunArgs],
@@ -645,7 +720,7 @@ class Agent:
 
                 # This *could be* the final message.
                 if isinstance(output, Message):
-                    yield copy(output)
+                    yield output
                     response = output
 
             if response is None:
@@ -662,35 +737,28 @@ class Agent:
             ):
                 return
 
-            results = []
-            last_tool_call = None
-            for tool_call in response.content:
-                if isinstance(tool_call, ToolCall):
-                    result = None
-                    last_tool_call = tool_call
-                    async for intermediate in self.invoke_tool(tool_call, messages):
-                        if isinstance(
-                            intermediate, (ThoughtBlock, MessageContent, Message)
-                        ):
-                            yield intermediate
-                        result = intermediate
-                    if result is None:
-                        raise ValueError(
-                            f"Tool call {tool_call} did not yield a result."
-                        )
-                    results.append(result)
+            all_tool_calls = [
+                item for item in response.content if isinstance(item, ToolCall)
+            ]
+            last_tool_call = all_tool_calls[-1] if all_tool_calls else None
+            tool_results: Sequence[ToolResult] = []
+            async for intermediate in self.invoke_tools(all_tool_calls, messages):
+                if isinstance(intermediate, Sequence):
+                    tool_results = intermediate
+                else:
+                    yield intermediate
+
             # Hook to modify tool results or perform post-tool processing.
             async for output in self.modify_tool_results(
                 messages=messages,
                 tool_call_message=response,
-                tool_results=results,
+                tool_results=tool_results,
             ):
                 yield output
 
             # Add the tool results message to the thread.
-            messages.append(response)
-            result_message = UserMessage(results)
-            messages.append(result_message)
+            result_message = UserMessage(tool_results)
+            messages = (*messages, response, result_message)
             yield result_message
 
             if isinstance(last_tool_call, ToolCall) and any(
