@@ -1,4 +1,4 @@
-from collections.abc import AsyncGenerator
+from collections.abc import AsyncGenerator, Sequence
 from dataclasses import dataclass
 
 from flexai.capability import Capability
@@ -13,6 +13,6 @@ class TruncateMessages(Capability):
     max_messages: int
 
     async def modify_messages(
-        self, messages: list[Message]
-    ) -> AsyncGenerator[MessageContent | list[Message], None]:
+        self, messages: Sequence[Message]
+    ) -> AsyncGenerator[MessageContent | Sequence[Message], None]:
         yield messages[-self.max_messages :]
