@@ -56,8 +56,8 @@ class Capability:
         yield tools
 
     async def modify_messages(
-        self, messages: list[Message]
-    ) -> AsyncGenerator[MessageContent | Message | list[Message], None]:
+        self, messages: Sequence[Message]
+    ) -> AsyncGenerator[MessageContent | Message | Sequence[Message], None]:
         """Modify the conversation messages before sending them to the LLM.
 
         This method can be used to add, remove, or alter messages in the
@@ -72,7 +72,7 @@ class Capability:
         yield messages
 
     async def modify_response(
-        self, messages: list[Message], response: AIMessage
+        self, messages: Sequence[Message], response: AIMessage
     ) -> AsyncGenerator[MessageContent | AIMessage, None]:
         """Modify the AI-generated response before sending it to the user.
 
@@ -90,9 +90,9 @@ class Capability:
 
     async def modify_tool_results(
         self,
-        messages: list[Message],
+        messages: Sequence[Message],
         tool_call_message: Message[Sequence[MessageContent] | str],
-        tool_results: list[MessageContent],
+        tool_results: Sequence[MessageContent],
     ) -> AsyncGenerator[Message | MessageContent, None]:
         """Hook to modify the tool calls and results before sending them to the user.
 
