@@ -17,7 +17,7 @@ class DefaultClient(Client):
 
     async def get_chat_response(
         self,
-        messages: list[Message],
+        messages: Sequence[Message],
         system: str | SystemMessage = "",
         tools: Sequence[Tool] | None = None,
         **input_args: Unpack[PartialAgentRunArgs],
@@ -28,7 +28,7 @@ class DefaultClient(Client):
 
     async def stream_chat_response(
         self,
-        messages: list[Message],
+        messages: Sequence[Message],
         system: str | SystemMessage = "",
         tools: Sequence[Tool] | None = None,
         **input_args: Unpack[PartialAgentRunArgs],
