@@ -1,11 +1,11 @@
 from __future__ import annotations
 
 import base64
-import json
+import itertools
 import os
 import time
 from collections.abc import AsyncGenerator, Sequence
-from dataclasses import dataclass, field
+from dataclasses import InitVar, dataclass, field
 from typing import Any, Unpack
 
 from anthropic import AsyncAnthropic, AsyncAnthropicBedrock
@@ -75,6 +75,9 @@ class AnthropicClient(Client):
         default_factory=lambda: AsyncAnthropic(api_key=os.getenv("ANTHROPIC_API_KEY"))
     )
 
+    # The base URL to use for interacting with the model.
+    base_url: InitVar[str | None] = None
+
     # The model to use for generating responses.
     model: str = os.getenv("ANTHROPIC_MODEL", "claude-3-5-sonnet-latest")
 
@@ -84,8 +87,19 @@ class AnthropicClient(Client):
     # Whether to cache messages or not.
     cache_messages: bool = False
 
-    # Extra headers to include in the request.
-    extra_headers: dict = field(default_factory=dict)
+    # Extra headers to include in every request.
+    default_extra_headers: dict[str, str] = field(default_factory=dict)
+
+    def __post_init__(self, base_url: str | None):
+        if base_url is not None:
+            object.__setattr__(
+                self,
+                "client",
+                AsyncAnthropic(
+                    base_url=base_url,
+                    api_key=os.getenv("ANTHROPIC_API_KEY"),
+                ),
+            )
 
     def _complete_args_with_defaults(
         self, **run_args: Unpack[PartialAgentRunArgs]
@@ -97,11 +111,15 @@ class AnthropicClient(Client):
             defaults["force_tool"] = True
         if run_args.get("thinking_budget") is None:
             defaults["thinking_budget"] = self.default_thinking_budget
+        defaults["extra_headers"] = {
+            **self.default_extra_headers,
+            **defaults["extra_headers"],
+        }
         return defaults
 
     async def get_chat_response(
         self,
-        messages: list[Message],
+        messages: Sequence[Message],
         system: str | SystemMessage = "",
         tools: Sequence[Tool] | None = None,
         **input_args: Unpack[PartialAgentRunArgs],
@@ -160,7 +178,7 @@ class AnthropicClient(Client):
 
     async def stream_chat_response(
         self,
-        messages: list[Message],
+        messages: Sequence[Message],
         system: str | SystemMessage = "",
         tools: Sequence[Tool] | None = None,
         **input_args: Unpack[PartialAgentRunArgs],
@@ -294,7 +312,7 @@ class AnthropicClient(Client):
 
     def _get_params(
         self,
-        messages: list[Message],
+        messages: Sequence[Message],
         system: str | SystemMessage,
         tools: Sequence[Tool] | None,
         stream: bool,
@@ -322,6 +340,7 @@ class AnthropicClient(Client):
         thinking_budget = run_args["thinking_budget"]
         allow_tool = run_args["allow_tool"]
         force_tool = run_args["force_tool"]
+        extra_headers = run_args["extra_headers"]
 
         thinking_args = (
             {"thinking": {"type": "disabled"}}
@@ -336,7 +355,7 @@ class AnthropicClient(Client):
             "messages": self._format_content(messages, allow_tool),
             "model": self.model,
             "temperature": temperature,
-            "extra_headers": self.extra_headers,
+            "extra_headers": extra_headers,
             **thinking_args,
         }
 
@@ -354,8 +373,16 @@ class AnthropicClient(Client):
             params["tools"] = sorted(
                 [self.format_tool(tool) for tool in tools], key=lambda x: x["name"]
             )
+            force_tool = False
+            # Default to disabling parallel tool use.
+            tool_choice: dict[str, Any] = {
+                "type": "auto",
+                "disable_parallel_tool_use": True,
+            }
             if force_tool:
-                params["tool_choice"] = {"type": "any"}
+                tool_choice["type"] = "any"
+            params["tool_choice"] = tool_choice
+
         else:
             params["tools"] = []
 
@@ -367,7 +394,7 @@ class AnthropicClient(Client):
 
     async def _get_structured_response(
         self,
-        messages: list[Message],
+        messages: Sequence[Message],
         model_to_use: type[BaseModel],
         system: str | SystemMessage = "",
         tools: Sequence[Tool] | None = None,
@@ -522,82 +549,120 @@ Do not begin your response with ```json or end it with ```.
         contents: Sequence[MessageContent],
         allow_tool: bool = True,
     ) -> list[dict[str, Any]]:
-        return [
-            (
-                {
-                    "type": "text",
-                    "text": content.text,
-                }
-                if not content.cache
-                else {
-                    "type": "text",
-                    "text": content.text,
-                    "cache_control": {"type": "ephemeral"},
-                }
-            )
-            if isinstance(content, TextBlock)
-            else {
-                "type": "image",
-                "source": {
-                    "type": "base64",
-                    "media_type": content.mime_type,
-                    "data": base64.b64encode(content.image).decode("utf-8"),
-                },
-            }
-            if isinstance(content, ImageBlock)
-            else (
-                {
-                    "type": "tool_use",
-                    "id": content.id,
-                    "name": content.name,
-                    "input": content.input,
-                }
-                if allow_tool
-                else {
-                    "type": "text",
-                    "text": json.dumps(
+        return list(
+            itertools.chain.from_iterable(
+                [
+                    (
                         {
-                            "tool_use": {
-                                "id": content.id,
-                                "name": content.name,
-                                "input": content.input,
-                            }
+                            "type": "text",
+                            "text": content.text,
                         }
-                    ),
-                }
-            )
-            if isinstance(content, ToolCall)
-            else (
-                {
-                    "type": "tool_result",
-                    "tool_use_id": content.tool_call_id,
-                    "content": cls._format_tool_result(content.result),
-                    "is_error": content.is_error,
-                }
-                if allow_tool
-                else {
-                    "type": "text",
-                    "text": json.dumps(
-                        {
-                            "tool_result": {
-                                "tool_call_id": content.tool_call_id,
-                                "result": content.result,
-                                "is_error": content.is_error,
-                            }
+                        if not content.cache
+                        else {
+                            "type": "text",
+                            "text": content.text,
+                            "cache_control": {"type": "ephemeral"},
                         }
-                    ),
-                }
-            )
-            if isinstance(content, ToolResult)
-            else (
-                (_ for _ in ()).throw(
-                    TypeError(
-                        f"Tried to send {content} to anthropic, which is of an unsupported type."
                     )
+                ]
+                if isinstance(content, TextBlock)
+                else [
+                    {
+                        "type": "image",
+                        "source": {
+                            "type": "base64",
+                            "media_type": content.mime_type,
+                            "data": base64.b64encode(content.image).decode("utf-8"),
+                        },
+                    }
+                ]
+                if isinstance(content, ImageBlock)
+                else (
+                    [
+                        {
+                            "type": "tool_use",
+                            "id": content.id,
+                            "name": content.name,
+                            "input": content.input,
+                        }
+                    ]
+                    if allow_tool
+                    else [
+                        {
+                            "type": "text",
+                            "text": str(
+                                {
+                                    "tool_use": {
+                                        "id": content.id,
+                                        "name": content.name,
+                                        "input": content.input,
+                                    }
+                                }
+                            ),
+                        }
+                    ]
                 )
+                if isinstance(content, ToolCall)
+                else (
+                    [
+                        {
+                            "type": "tool_result",
+                            "tool_use_id": content.tool_call_id,
+                            "content": cls._format_tool_result(content.result),
+                            "is_error": content.is_error,
+                        }
+                    ]
+                    if allow_tool
+                    else [
+                        {
+                            "type": "text",
+                            "text": str(
+                                {
+                                    "tool_result": {
+                                        "tool_call_id": content.tool_call_id,
+                                        "result": (
+                                            content.result
+                                            if not isinstance(
+                                                content.result, ImageBlock
+                                            )
+                                            else "Result was an image, included in the following message.",
+                                        ),
+                                        "is_error": content.is_error,
+                                    },
+                                }
+                            ),
+                        },
+                        *(
+                            [
+                                {
+                                    "type": "image",
+                                    "source": {
+                                        "type": "base64",
+                                        "media_type": content.result.mime_type,
+                                        "data": base64.b64encode(
+                                            content.result.image
+                                        ).decode("utf-8"),
+                                    },
+                                }
+                            ]
+                            if isinstance(content.result, ImageBlock)
+                            else []
+                        ),
+                    ]
+                )
+                if isinstance(content, ToolResult)
+                else (
+                    [
+                        (_ for _ in ()).throw(
+                            TypeError(
+                                f"Tried to send {content} to anthropic, which is of an unsupported type."
+                            )
+                        )
+                    ]
+                )
+                for content in contents
             )
-            for content in contents
-        ]
+        )
 
     @classmethod
     def _format_content(
