@@ -41,6 +41,9 @@ class AgentRunArgs(TypedDict):
     # Whether to allow image responses from the model.
     allow_image: bool
 
+    # Extra run headers
+    extra_headers: dict[str, str]
+
 
 class PartialAgentRunArgs(TypedDict, total=False):
     """AgentRunArgs, but some fields are allowed to be unspecified."""
@@ -54,6 +57,7 @@ class PartialAgentRunArgs(TypedDict, total=False):
     raise_on_null_content: bool
     allow_text: bool
     allow_image: bool
+    extra_headers: dict[str, str]
 
 
 def with_defaults(**kwargs: Unpack[PartialAgentRunArgs]) -> AgentRunArgs:
@@ -75,6 +79,7 @@ def with_defaults(**kwargs: Unpack[PartialAgentRunArgs]) -> AgentRunArgs:
         "raise_on_null_content": False,
         "allow_text": True,
         "allow_image": False,
+        "extra_headers": {},
     }
     return {**defaults, **kwargs}
 
@@ -171,7 +176,7 @@ class Client(ABC):
     @abstractmethod
     async def get_chat_response(
         self,
-        messages: list[Message],
+        messages: Sequence[Message],
         system: str | SystemMessage = "",
         tools: Sequence[Tool] | None = None,
         **input_args: Unpack[PartialAgentRunArgs],
@@ -196,7 +201,7 @@ class Client(ABC):
     @abstractmethod
     def stream_chat_response(
         self,
-        messages: list[Message],
+        messages: Sequence[Message],
         system: str | SystemMessage = "",
         tools: Sequence[Tool] | None = None,
         **input_args: Unpack[PartialAgentRunArgs],
