@@ -3,7 +3,6 @@
 from __future__ import annotations
 
 from dataclasses import dataclass
-from typing import Any
 
 
 @dataclass
@@ -14,7 +13,6 @@ class LLMException(Exception):  # noqa: N818
     provider: str
     original_exception: Exception | None = None
     status_code: int | None = None
-    response_data: dict[str, Any] | None = None
 
     def __str__(self) -> str:
         result = f"[{self.provider}] {self.message}"
@@ -135,7 +133,7 @@ STATUS_CODE_MAPPINGS: dict[int, type[LLMException]] = {
 # Provider-specific exception extractors
 def extract_openai_details(
     exception: Exception,
-) -> tuple[int | None, dict[str, Any] | None, str]:
+) -> tuple[int | None, str]:
     """Extract status code, response data and message from OpenAI exceptions.
 
     Args:
@@ -144,25 +142,22 @@ def extract_openai_details(
     Returns:
         A tuple containing:
             - status_code: The HTTP status code if available, otherwise None.
-            - response_data: The response data as a dictionary if available, otherwise None.
             - message: A string message describing the error.
     """
     from openai import APIError
 
     if isinstance(exception, APIError):
-        body = getattr(exception, "body", None)
         return (
             getattr(exception, "status_code", None),
-            getattr(body, "__dict__", None),
             exception.message,
         )
 
-    return None, exception.__dict__, str(exception)
+    return None, str(exception)
 
 
 def extract_anthropic_details(
     exception: Exception,
-) -> tuple[int | None, dict[str, Any] | None, str]:
+) -> tuple[int | None, str]:
     """Extract status code, response data and message from Anthropic exceptions.
 
     Args:
@@ -177,19 +172,17 @@ def extract_anthropic_details(
     from anthropic import APIError
 
     if isinstance(exception, APIError):
-        body = getattr(exception, "body", None)
         return (
             getattr(exception, "status_code", None),
-            getattr(body, "__dict__", None),
             exception.message,
         )
 
-    return None, exception.__dict__, str(exception)
+    return None, str(exception)
 
 
 def extract_gemini_details(
     exception: Exception,
-) -> tuple[int | None, dict[str, Any] | None, str]:
+) -> tuple[int | None, str]:
     """Extract status code, response data and message from Gemini exceptions.
 
     Args:
@@ -208,19 +201,14 @@ def extract_gemini_details(
         response_json = exception.details
         return (
             exception._get_code(response_json),
-            response_json,
             exception._get_message(response_json),
         )
 
     # Handle HTTPX errors which Gemini sometimes raises directly
     if isinstance(exception, httpx.HTTPStatusError):
-        try:
-            response_data = exception.response.json()
-        except Exception:
-            response_data = {"error": str(exception.response)}
-        return exception.response.status_code, response_data, str(exception)
+        return exception.response.status_code, str(exception)
 
-    return None, exception.__dict__, str(exception)
+    return None, str(exception)
 
 
 # Map of provider to extraction function
@@ -246,8 +234,8 @@ def map_exception(exception: Exception, provider: str) -> Exception:
         return exception
 
     # Extract details using provider-specific extractor
-    extractor = PROVIDER_EXTRACTORS.get(provider, lambda e: (None, None, str(e)))
-    status_code, response_data, message = extractor(exception)
+    extractor = PROVIDER_EXTRACTORS.get(provider, lambda e: (None, str(e)))
+    status_code, message = extractor(exception)
 
     # Get the appropriate exception class based on status code
     exception_class = STATUS_CODE_MAPPINGS.get(status_code) if status_code else None
@@ -266,5 +254,4 @@ def map_exception(exception: Exception, provider: str) -> Exception:
         provider=provider,
         original_exception=exception,
         status_code=status_code,
-        response_data=response_data,
     )
