@@ -133,13 +133,16 @@ class GeminiClient(Client):
     _client: genai.client.AsyncClient | None = None
 
     # The base URL for the Gemini API.
-    base_url: InitVar[str] = field(
+    base_url: str = field(
         default=os.environ.get(
             "GEMINI_BASE_URL",
             "https://www.googleapis.com/auth/generative-language",
         )
     )
 
+    # Extra headers to include in the request.
+    extra_headers: dict[str, str] = field(default_factory=dict)
+
     # The model to use for the client.
     model: str = os.getenv("GEMINI_MODEL") or "gemini-2.5-pro-preview-06-05"
 
@@ -214,13 +217,21 @@ class GeminiClient(Client):
 
     @staticmethod
     @functools.lru_cache
-    def _get_vertex_client(location: str, project_id: str, credential_file_path: str):
+    def _get_vertex_client(
+        location: str,
+        project_id: str,
+        credential_file_path: str,
+        base_url: str,
+        extra_headers: tuple[tuple[str, str]],
+    ):
         """Get a cached Vertex AI client instance.
 
         Args:
             location: The location/region for Vertex AI.
             project_id: Google Cloud project ID.
             credential_file_path: Path to service account credentials file.
+            base_url: The base URL to use for interacting with the model.
+            extra_headers: Any extra headers.
 
         Returns:
             A cached AsyncClient configured for Vertex AI.
@@ -236,6 +247,10 @@ class GeminiClient(Client):
                 ),
                 location=location,
                 project=project_id,
+                http_options=types.HttpOptionsDict(
+                    base_url=base_url,
+                    headers=dict(extra_headers),
+                ),
             )
         )
 
@@ -252,7 +267,7 @@ class GeminiClient(Client):
         """
         return genai.client.AsyncClient(api_client=BaseApiClient(api_key=api_key))
 
-    def __post_init__(self, api_key, base_url, **kwargs):
+    def __post_init__(self, api_key, **kwargs):
         use_vertex = kwargs.get("use_vertex", self.use_vertex)
         credential_file_path = kwargs.get("credential_file_path", "")
 
@@ -264,6 +279,8 @@ class GeminiClient(Client):
                     location=self.location,
                     project_id=self.project_id,
                     credential_file_path=credential_file_path,
+                    base_url=self.base_url,
+                    extra_headers=tuple(self.extra_headers.items()),
                 ),
             )
         else:
@@ -355,17 +372,52 @@ class GeminiClient(Client):
                 mime_type=part_object.inline_data.mime_type,
             )
 
+    @classmethod
+    def _format_tool_call(
+        cls,
+        content: ToolCall,
+        allow_tool: bool,
+    ):
+        if allow_tool:
+            return {
+                "functionCall": {
+                    "id": content.id,
+                    "name": content.name,
+                    "args": content.input,
+                }
+            }
+        return {"text": str(content)}
+
+    @classmethod
+    def _format_tool_result(
+        cls,
+        content: ToolResult,
+        name_context: dict,
+        allow_tool: bool,
+    ):
+        if allow_tool:
+            return {
+                "functionResponse": {
+                    "id": content.tool_call_id,
+                    "name": name_context[content.tool_call_id],
+                    "response": content.result,
+                }
+            }
+        return {"text": str(content)}
+
     @classmethod
     def _format_message_content(
         cls,
         content: str | MessageContent | Sequence[MessageContent],
         name_context: dict,
+        allow_tool: bool,
     ):
         """Format message content for Gemini API.
 
         Args:
             content: The message content to format (string, MessageContent, or sequence).
             name_context: Dictionary to track tool call names by ID.
+            allow_tool: Whether to allow tool-related content in the formatting.
 
         Returns:
             Formatted content structure for Gemini API.
@@ -378,7 +430,9 @@ class GeminiClient(Client):
 
         if isinstance(content, Sequence):
             formatted_contents = [
-                cls._format_message_content(item, name_context=name_context)
+                cls._format_message_content(
+                    item, name_context=name_context, allow_tool=allow_tool
+                )
                 for item in content
             ]
             # Just a list flatten. I don't like itertools.chain.from_iterable personally
@@ -403,18 +457,19 @@ class GeminiClient(Client):
             }
         if isinstance(content, DataBlock):
             return [
-                cls._format_message_content(item, name_context=name_context)
+                cls._format_message_content(
+                    item, name_context=name_context, allow_tool=allow_tool
+                )
                 for item in content.into_text_and_image_blocks()
             ]
+
         if isinstance(content, ToolCall):
             name_context[content.id] = content.name
-            return {
-                "functionCall": {
-                    "id": content.id,
-                    "name": content.name,
-                    "args": content.input,
-                }
-            }
+            return cls._format_tool_call(
+                content,
+                allow_tool=allow_tool,
+            )
+
         if isinstance(content, ToolResult):
             formatted_result = content.result
 
@@ -425,24 +480,25 @@ class GeminiClient(Client):
 
             if isinstance(formatted_result, ImageBlock):
                 return [
-                    {
-                        "functionResponse": {
-                            "id": content.tool_call_id,
-                            "name": name_context[content.tool_call_id],
-                            "response": {
-                                "text": "The result of this tool is an image. The next part will contain this image.",
-                            },
-                        },
-                    },
+                    cls._format_tool_result(
+                        content.with_result(
+                            {
+                                "text": "The result of this tool is an image. The next part will contain this image."
+                            }
+                        ),
+                        name_context=name_context,
+                        allow_tool=allow_tool,
+                    ),
                     cls._format_message_content(
                         formatted_result,
                         name_context=name_context,
+                        allow_tool=allow_tool,
                     ),
                 ]
 
             try:
                 formatted_result = cls._format_message_content(
-                    formatted_result, name_context=name_context
+                    formatted_result, name_context=name_context, allow_tool=allow_tool
                 )
             except ValueError:
                 formatted_result = {"result": str(formatted_result)}
@@ -451,18 +507,18 @@ class GeminiClient(Client):
                 raise ValueError(
                     f"Tool call {content.tool_call_id} not found in context, but a result for it was found."
                 )
-            return {
-                "functionResponse": {
-                    "id": content.tool_call_id,
-                    "name": name_context[content.tool_call_id],
-                    "response": formatted_result,
-                }
-            }
+
+            return cls._format_tool_result(
+                content.with_result(formatted_result),
+                name_context=name_context,
+                allow_tool=allow_tool,
+            )
+
         raise ValueError(f"Unsupported content type: {type(content)}")
 
     def _get_params(
         self,
-        messages: list[Message],
+        messages: Sequence[Message],
         system: str | SystemMessage,
         tools: Sequence[Tool] | None,
         stream: bool,
@@ -484,6 +540,7 @@ class GeminiClient(Client):
         """
         # Extract run arguments that we care about
         force_tool = run_args["force_tool"]
+        allow_tool = run_args["allow_tool"]
         include_thoughts = run_args["include_thoughts"]
         model = run_args["model"]
         thinking_budget = run_args["thinking_budget"]
@@ -496,7 +553,9 @@ class GeminiClient(Client):
             {
                 "role": "model" if message.role == "assistant" else "user",
                 "parts": self._format_message_content(
-                    message.content, name_context=name_context
+                    message.content,
+                    name_context=name_context,
+                    allow_tool=allow_tool,
                 ),
             }
             for message in messages
@@ -507,7 +566,9 @@ class GeminiClient(Client):
 
         formatted_system = json.dumps(
             self._format_message_content(
-                system.normalize().content, name_context=name_context
+                system.normalize().content,
+                name_context=name_context,
+                allow_tool=allow_tool,
             )
         )
 
@@ -580,7 +641,7 @@ class GeminiClient(Client):
 
     async def get_chat_response(
         self,
-        messages: list[Message],
+        messages: Sequence[Message],
         system: str | SystemMessage = "",
         tools: Sequence[Tool] | None = None,
         **input_args: Unpack[PartialAgentRunArgs],
@@ -665,7 +726,7 @@ class GeminiClient(Client):
 
     async def stream_chat_response(
         self,
-        messages: list[Message],
+        messages: Sequence[Message],
         system: str | SystemMessage = "",
         tools: Sequence[Tool] | None = None,
         **input_args: Unpack[PartialAgentRunArgs],
