@@ -33,6 +33,7 @@ from flexai.message import (
     MessageContent,
     SystemMessage,
     TextBlock,
+    ThoughtBlock,
     ToolCall,
     ToolResult,
     Usage,
@@ -74,10 +75,14 @@ def get_usage_block(usage: CompletionUsage | None) -> Usage:
     Returns:
         A Usage object containing input tokens, output tokens, and generation time.
     """
+    cache_read = 0
+    if usage and usage.prompt_tokens_details:
+        cache_read = usage.prompt_tokens_details.cached_tokens or 0
     return (
         Usage(
             input_tokens=usage.prompt_tokens,
             output_tokens=usage.completion_tokens,
+            cache_read_tokens=cache_read,
         )
         if usage is not None
         else Usage(input_tokens=0, output_tokens=0)
@@ -106,6 +111,12 @@ class OpenAIClient(Client):
     # The model to use for generating responses.
     model: str = os.getenv("OPENAI_MODEL", "gpt-4o-mini")
 
+    # Extra headers to include in all requests
+    default_extra_headers: dict[str, str] = field(default_factory=dict)
+
+    # Extra body arguments in the request.
+    extra_body: dict[str, Any] = field(default_factory=dict)
+
     def __post_init__(self, api_key, base_url, **kwargs):
         object.__setattr__(
             self,
@@ -121,11 +132,15 @@ class OpenAIClient(Client):
             defaults["temperature"] = 1.0
         if run_args.get("thinking_budget") is None:
             defaults["thinking_budget"] = self.default_thinking_budget
+        defaults["extra_headers"] = {
+            **self.default_extra_headers,
+            **defaults["extra_headers"],
+        }
         return defaults
 
     async def get_chat_response(
         self,
-        messages: list[Message],
+        messages: Sequence[Message],
         system: str | SystemMessage = "",
         tools: Sequence[Tool] | None = None,
         **input_args: Unpack[PartialAgentRunArgs],
@@ -154,42 +169,57 @@ class OpenAIClient(Client):
         response = await self.client.chat.completions.create(**params, stream=False)
         generation_time = time.monotonic() - start
 
-        # Parse out the tool uses from the response.
         message = response.choices[0].message
-        tool_uses = [
-            tool_call
-            for tool_use_block in (message.tool_calls or [])
-            if not isinstance(tool_use_block, ChatCompletionMessageCustomToolCall)
-            if (tool_call := get_tool_call(tool_use_block)) is not None
-        ]
+        formatted_content_parts = []
+
+        if message.content:
+            formatted_content_parts.append(TextBlock(text=message.content))
+
+        # Parse out the tool uses from the response.
+        formatted_content_parts.extend(
+            [
+                tool_call
+                for tool_use_block in (message.tool_calls or [])
+                if not isinstance(tool_use_block, ChatCompletionMessageCustomToolCall)
+                if (tool_call := get_tool_call(tool_use_block)) is not None
+            ]
+        )
+
         usage = get_usage_block(response.usage)
         usage.generation_time = generation_time
 
-        # Get the content to return.
-        content_to_return = tool_uses or message.content or []
         return AIMessage(
-            content=content_to_return,
+            content=formatted_content_parts,
             usage=usage,
         )
 
     async def stream_chat_response(
         self,
-        messages: list[Message],
+        messages: Sequence[Message],
         system: str | SystemMessage = "",
         tools: Sequence[Tool] | None = None,
         **input_args: Unpack[PartialAgentRunArgs],
     ) -> AsyncGenerator[MessageContent | AIMessage, None]:
-        run_args = self._complete_args_with_defaults(**input_args)
+        from openai.lib._parsing import type_to_response_format_param
 
-        stream = await self.client.chat.completions.create(
+        run_args = self._complete_args_with_defaults(**input_args)
+        complete_args = {
             **self._get_params(
                 messages=messages,
                 system=system,
                 tools=tools,
                 **run_args,
             ),
-            stream=True,
-        )
+            "stream": True,
+        }
+        model_to_use = run_args["model"]
+        if model_to_use is not None:
+            if not issubclass(model_to_use, BaseModel):
+                raise TypeError("model_to_use must be a subclass of BaseModel.")
+            complete_args["response_format"] = type_to_response_format_param(
+                model_to_use
+            )
+        stream = await self.client.chat.completions.create(**complete_args)
         current_tool_call: ToolCall | None = None
         current_text_block: TextBlock | None = None
 
@@ -197,8 +227,16 @@ class OpenAIClient(Client):
             if not isinstance(chunk.choices, list) or not chunk.choices:
                 continue
             content = chunk.choices[0].delta.content
+            extra_content = getattr(chunk.choices[0].delta, "extra_content", None)
 
             if content:
+                if (
+                    extra_content
+                    and isinstance(extra_content, dict)
+                    and extra_content.get("google", {}).get("thought") is True
+                ):
+                    yield ThoughtBlock(text=content)
+                    continue
                 yield TextBlock(text=content)
                 if not current_text_block:
                     current_text_block = TextBlock(text="")
@@ -238,7 +276,7 @@ class OpenAIClient(Client):
 
     def _get_params(
         self,
-        messages: list[Message],
+        messages: Sequence[Message],
         system: str | SystemMessage,
         tools: Sequence[Tool] | None,
         **run_args: Unpack[AgentRunArgs],
@@ -256,7 +294,9 @@ class OpenAIClient(Client):
             The common params to send to the model.
         """
         # Extract the run arguments we care about.
+        force_tool = run_args["force_tool"]
         temperature = run_args["temperature"]
+        extra_headers = run_args["extra_headers"]
 
         if isinstance(system, str):
             system = SystemMessage(system)
@@ -277,17 +317,23 @@ class OpenAIClient(Client):
             "reasoning_effort": reasoning_effort,
         }
 
+        if extra_headers:
+            kwargs["extra_headers"] = extra_headers
+
+        if self.extra_body:
+            kwargs["extra_body"] = self.extra_body
+
         # If tools are provided, force the model to use them (for now).
         if tools:
             kwargs["tools"] = [self.format_tool(tool) for tool in tools]
-            kwargs["tool_choice"] = "required"
+            kwargs["tool_choice"] = "required" if force_tool else "auto"
             kwargs["parallel_tool_calls"] = False
 
         return kwargs
 
     async def _get_structured_response(
         self,
-        messages: list[Message],
+        messages: Sequence[Message],
         model_to_use: type[BASE_MODEL],
         system: str | SystemMessage = "",
         tools: Sequence[Tool] | None = None,
