@@ -6,7 +6,7 @@ import inspect
 import types
 from collections.abc import Callable
 from dataclasses import dataclass
-from typing import Literal, get_type_hints
+from typing import Literal, get_origin, get_type_hints
 
 JSON_SCHEMA_TYPES = Literal[
     "string", "integer", "number", "boolean", "array", "object", "null"
@@ -38,7 +38,7 @@ ToolType = (
 def format_primitive_type(
     arg_type,
 ) -> JSON_SCHEMA_TYPES | UNSPECIFIED_JSON_SCHEMA_TYPE:
-    return TYPE_MAP.get(arg_type, TYPE_UNSPECIFIED)
+    return TYPE_MAP.get(get_origin(arg_type) or arg_type, TYPE_UNSPECIFIED)
 
 
 def format_type(arg_type) -> ToolType:
@@ -47,6 +47,7 @@ def format_type(arg_type) -> ToolType:
             format_primitive_type(sub_type) for sub_type in arg_type.__args__
         ]
         return tuple(sub_types)
+
     return format_primitive_type(arg_type)
 
 
