@@ -447,7 +447,7 @@ async def agent_integration_template(client):
 
 async def multi_turn_template(client):
     """Generic template for testing multi-turn conversations."""
-    response1, response2, messages = await run_multi_turn_conversation(
+    response1, response2, _messages = await run_multi_turn_conversation(
         client,
         TestMessages.CONVERSATION_START[0],
         TestMessages.CONVERSATION_FOLLOWUP[0],
