@@ -8,7 +8,6 @@ from flexai.message import (
     AIMessage,
     Message,
     SystemMessage,
-    TextBlock,
     ToolCall,
     ToolResult,
     UserMessage,
@@ -93,17 +92,15 @@ async def test_invoke_tool():
 
     agent = Agent(tools=[sync_tool, async_tool])
 
-    async for message in agent.invoke_tool(
+    sync_result = await agent.invoke_tool(
         ToolCall(id="1", name="sync_tool", input={"x": 2}), []
-    ):
-        sync_result = message
+    )
     assert sync_result.result == 4
     assert not sync_result.is_error
 
-    async for message in agent.invoke_tool(
+    async_result = await agent.invoke_tool(
         ToolCall(id="1", name="async_tool", input={"x": 2}), []
-    ):
-        async_result = message
+    )
     assert async_result.result == 6
     assert not async_result.is_error
 
@@ -142,8 +139,7 @@ async def test_multiple_tool_execution():
 
     results = []
     for tool_call in response.content:
-        async for message in agent.invoke_tool(tool_call, []):
-            final_response = message
+        final_response = await agent.invoke_tool(tool_call, [])
         results.append(final_response)
     assert len(results) == 2
     assert results[0].result == 10
@@ -157,8 +153,7 @@ async def test_error_handling_in_tools():
 
     agent = Agent(tools=[faulty_tool])
     tool_call = ToolCall(id="1", name="faulty_tool", input={})
-    async for message in agent.invoke_tool(tool_call, []):
-        result = message
+    result = await agent.invoke_tool(tool_call, [])
 
     assert result.is_error
     assert "Tool error" in str(result.result)
@@ -202,7 +197,7 @@ async def test_capability_chaining():
 async def test_agent_stream():
     class TestAgent(Agent):
         async def invoke_tool(self, tool_call, messages):
-            yield ToolResult(
+            return ToolResult(
                 tool_call_id=tool_call.id,
                 result="Tool result",
                 execution_time=0.1,
@@ -253,8 +248,7 @@ async def test_rate_limiting_simulation():
     agent = Agent(tools=[slow_tool])
     tool_call = ToolCall(id="1", name="slow_tool", input={})
 
-    async for message in agent.invoke_tool(tool_call, []):
-        result = message
+    result = await agent.invoke_tool(tool_call, [])
 
     assert result.execution_time >= 0.1
     assert result.result == "Slow result"
@@ -265,14 +259,11 @@ async def test_invoke_tool_missing():
     agent = Agent(tools=[])
     tool_call = ToolCall(id="1", name="nonexistent_tool", input={})
 
-    result = None
-    async for message in agent.invoke_tool(tool_call, []):
-        result = message
+    result = await agent.invoke_tool(tool_call, [])
 
     assert isinstance(result, ToolResult)
     assert result.is_error
-    assert isinstance(result.result, TextBlock)
-    assert "not available" in result.result.text.lower()
+    assert "not available" in str(result.result).lower()
 
 
 class MockClient:
@@ -691,3 +682,76 @@ async def test_get_tools_with_modifying_capability():
     tool_names = [tool.name for tool in result]
     assert "test_tool" in tool_names
     assert "additional_tool" in tool_names
+
+
+@pytest.mark.asyncio
+async def test_parallel_tool_execution():
+    """Test that parallel tool execution correctly collects all yielded items."""
+    from flexai.message import ToolUseChunk
+
+    async def streaming_tool_1(x):
+        """A tool that yields intermediate results."""
+        yield "tool1_chunk_1"
+        await asyncio.sleep(0.01)
+        yield "tool1_chunk_2"
+        await asyncio.sleep(0.01)
+        yield f"tool1_final_{x}"
+
+    async def streaming_tool_2(y):
+        """Another tool that yields intermediate results."""
+        yield "tool2_chunk_1"
+        await asyncio.sleep(0.015)
+        yield "tool2_chunk_2"
+        yield f"tool2_final_{y}"
+
+    async def non_streaming_tool(z):
+        """A tool that doesn't yield intermediate results."""
+        await asyncio.sleep(0.005)
+        return f"tool3_final_{z}"
+
+    agent = Agent(tools=[streaming_tool_1, streaming_tool_2, non_streaming_tool])
+
+    tool_calls = [
+        ToolCall(id="1", name="streaming_tool_1", input={"x": "a"}),
+        ToolCall(id="2", name="streaming_tool_2", input={"y": "b"}),
+        ToolCall(id="3", name="non_streaming_tool", input={"z": "c"}),
+    ]
+
+    # Collect all outputs
+    chunks = []
+    final_results = None
+
+    async for output in agent.invoke_tools(tool_calls, []):
+        if isinstance(output, list):
+            # This is the final list of ToolResults
+            final_results = output
+        else:
+            # This is an intermediate chunk or message
+            chunks.append(output)
+
+    # Verify we got all the chunks
+    chunk_contents = []
+    for chunk in chunks:
+        if isinstance(chunk, ToolUseChunk):
+            chunk_contents.append(chunk.content)
+
+    # Should have 2 chunks total (the first 2 from each streaming tool)
+    # The last yielded value becomes the final result, not a chunk
+    assert len(chunk_contents) == 4
+    assert "tool1_chunk_1" in chunk_contents
+    assert "tool1_chunk_2" in chunk_contents
+    assert "tool2_chunk_1" in chunk_contents
+    assert "tool2_chunk_2" in chunk_contents
+
+    # Verify we got all final results
+    assert final_results is not None
+    assert len(final_results) == 3
+
+    # Check that all final results are correct
+    results_by_id = {result.tool_call_id: result for result in final_results}
+    assert results_by_id["1"].result == "tool1_final_a"
+    assert results_by_id["2"].result == "tool2_final_b"
+    assert results_by_id["3"].result == "tool3_final_c"
+
+    # Verify none are errors
+    assert not any(result.is_error for result in final_results)
