@@ -1,6 +1,5 @@
 import json
 import os
-import sys
 from unittest.mock import AsyncMock, MagicMock, Mock, patch
 
 import pytest
@@ -22,11 +21,6 @@ from flexai.message import (
 )
 from flexai.tool import Tool
 
-# Mock openai module before any imports that might use it
-mock_openai = MagicMock()
-mock_openai.AsyncOpenAI = MagicMock()
-sys.modules["openai"] = mock_openai
-
 
 @pytest.fixture
 def mock_client():
